module NonConfig
  #
  # hash with overriden #method_missing
  #
  class NonHash < Hash
    # creates NonHash from Hash with recursive conversion of sub hashes
    def self.from_hash(hash)
      new_hash = {}
      hash.each do |k, v|
        new_hash[k] = case v
                      when Hash then from_hash(v)
                      when Array then convert_array_subhashes(v)
                      else v
                      end
      end

      new.merge(new_hash)
    end

    # recursively converts subhashes of `array` to `self.class`
    def self.convert_array_subhashes(array)
      array.map do |e|
        next convert_array_subhashes(e) if e.is_a? Array
        next from_hash(e) if e.is_a? Hash
        e
      end
    end

    # it allows constructions like non_hash.value1.value2
    def []=(key, value)
      value = self.class.from_hash(value) if value.is_a? Hash
      super(key, value)
    end

    def method_missing(m, *args)
      super unless args.empty?
      self[m.to_s] || self[m.to_sym]
    end

    # recursive conversion
    def to_hash
      result = {}

      each do |k, v|
        result[k] = case v
                    when self.class then v.to_hash
                    when Array then unconvert_array(v)
                    else v
                    end
      end

      result
    end

    #
    def to_yaml
      to_hash.to_yaml
    end

    private

    def unconvert_array(array)
      array.map do |e|
        next unconvert_array(e) if e.is_a? Array
        next e.to_hash if e.is_a? self.class
        e
      end
    end
  end
end
